/**********************************************************************
Copyright (c) 2006 Thomas Marti, Stefan Schmid and others. All rights reserved.
Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.

Contributors:
2006 Erik Bengtson - Original SpatialExpression implementation
    ...
**********************************************************************/
package org.datanucleus.store.expression.spatial;

import org.datanucleus.exceptions.NucleusException;
import org.datanucleus.store.mapped.expression.BooleanExpression;
import org.datanucleus.store.mapped.expression.BooleanLiteral;
import org.datanucleus.store.mapped.expression.Literal;
import org.datanucleus.store.mapped.expression.QueryExpression;
import org.datanucleus.store.mapped.expression.ScalarExpression;
import org.datanucleus.store.mapped.expression.StatementText;
import org.datanucleus.store.mapped.mapping.JavaTypeMapping;

/**
 * Representation of a geometry literal.
 */
public class GeometryLiteral extends ScalarExpression implements Literal
{
    private final Object value;

    public GeometryLiteral(QueryExpression qs, JavaTypeMapping mapping, Object value)
    {
        super(qs);

        this.mapping = mapping;
        this.value = value;

        if (mapping == null)
        {
            st.appendParameter(qs.getStoreManager().getMappingManager().getMappingWithDatastoreMapping(
                value.getClass(), false, false, qs.getClassLoaderResolver()), value);
        }
        else
        {
            st.appendParameter(mapping, value);
        }
    }

    public Object getValue()
    {
        return value;
    }

    public BooleanExpression eq(ScalarExpression expr)
    {
        if (expr instanceof GeometryLiteral)
        {
            return new BooleanLiteral(qs, mapping, value.equals(((GeometryLiteral) expr).value));
        }
        else
        {
            return super.eq(expr);
        }
    }

    public BooleanExpression noteq(ScalarExpression expr)
    {
        if (expr instanceof GeometryLiteral)
        {
            return new BooleanLiteral(qs, mapping, !value.equals(((GeometryLiteral) expr).value));
        }
        else
        {
            return super.noteq(expr);
        }
    }

    public StatementText toStatementText(int mode)
    {
        if (mode == ScalarExpression.PROJECTION)
        {
            throw new NucleusException("Literals of Geometry are not allowed to be projected in queries").setFatal();
        }
        return st;
    }

    public Object getRawValue()
    {
        // TODO Auto-generated method stub
        return null;
    }

    public void setRawValue(Object val)
    {
        // TODO Auto-generated method stub
        
    }
}