/**********************************************************************
 Copyright (c) 2006 Thomas Marti, Stefan Schmid and others. All rights reserved.
 Licensed under the Apache License, Version 2.0 (the "License");
 you may not use this file except in compliance with the License.
 You may obtain a copy of the License at

 http://www.apache.org/licenses/LICENSE-2.0

 Unless required by applicable law or agreed to in writing, software
 distributed under the License is distributed on an "AS IS" BASIS,
 WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 See the License for the specific language governing permissions and
 limitations under the License.

 Contributors:
 ...
 **********************************************************************/
package org.datanucleus.store.expression.spatial;

import java.util.ArrayList;

import org.datanucleus.store.mapped.expression.CharacterLiteral;
import org.datanucleus.store.mapped.expression.QueryExpression;
import org.datanucleus.store.mapped.expression.ScalarExpression;
import org.datanucleus.store.mapped.expression.StringExpression;
import org.datanucleus.store.mapped.mapping.JavaTypeMapping;

/**
 * Spatial expressions representations for PostGIS.
 */
public class PostgisSpatialExpression extends SpatialExpression
{
    public PostgisSpatialExpression(QueryExpression qs)
    {
        super(qs);
    }

    // ---- OGC SF functions ----//

    public ScalarExpression unionMethod(ScalarExpression geom1, ScalarExpression geom2)
    {
        return geometryFunction("GeomUnion", geom1, geom2, getGeomMapping(geom1));
    }

    public ScalarExpression numInteriorRingMethod(ScalarExpression geom)
    {
        JavaTypeMapping integerMapping = qs.getStoreManager().getMappingManager().getMappingWithDatastoreMapping(
            Integer.class, false, false, qs.getClassLoaderResolver());
        return numericFunction("NumInteriorRings", geom, integerMapping);
    }

    // ---- bounding box ----//

    public ScalarExpression bboxTestMethod(ScalarExpression geom1, ScalarExpression geom2)
    {
        return new SpatialBooleanExpression(geom1, "&&", geom2);
    }

    // ---- PostGIS ----//

    public ScalarExpression bboxOverlapsLeftMethod(ScalarExpression geom1, ScalarExpression geom2)
    {
        return new SpatialBooleanExpression(geom1, "&<", geom2);
    }

    public ScalarExpression bboxOverlapsRightMethod(ScalarExpression geom1, ScalarExpression geom2)
    {
        return new SpatialBooleanExpression(geom1, "&>", geom2);
    }

    public ScalarExpression bboxLeftMethod(ScalarExpression geom1, ScalarExpression geom2)
    {
        return new SpatialBooleanExpression(geom1, "<<", geom2);
    }

    public ScalarExpression bboxRightMethod(ScalarExpression geom1, ScalarExpression geom2)
    {
        return new SpatialBooleanExpression(geom1, ">>", geom2);
    }

    public ScalarExpression bboxOverlapsBelowMethod(ScalarExpression geom1, ScalarExpression geom2)
    {
        return new SpatialBooleanExpression(geom1, "&<|", geom2);
    }

    public ScalarExpression bboxOverlapsAboveMethod(ScalarExpression geom1, ScalarExpression geom2)
    {
        return new SpatialBooleanExpression(geom1, "|&>", geom2);
    }

    public ScalarExpression bboxBelowMethod(ScalarExpression geom1, ScalarExpression geom2)
    {
        return new SpatialBooleanExpression(geom1, "<<|", geom2);
    }

    public ScalarExpression bboxAboveMethod(ScalarExpression geom1, ScalarExpression geom2)
    {
        return new SpatialBooleanExpression(geom1, "|>>", geom2);
    }

    public ScalarExpression sameAsMethod(ScalarExpression geom1, ScalarExpression geom2)
    {
        return new SpatialBooleanExpression(geom1, "~=", geom2);
    }

    public ScalarExpression bboxWithinMethod(ScalarExpression geom1, ScalarExpression geom2)
    {
        return new SpatialBooleanExpression(geom1, "@", geom2);
    }

    public ScalarExpression bboxContainsMethod(ScalarExpression geom1, ScalarExpression geom2)
    {
        return new SpatialBooleanExpression(geom1, "~", geom2);
    }

    // ---- ----//

    protected ScalarExpression booleanFunction(String functionName, ScalarExpression expr)
    {
        ArrayList args = new ArrayList();
        args.add(expr);
        return new SpatialBooleanExpression(new StringExpression(functionName, args), new CharacterLiteral(qs, null, 't'));
    }

    protected ScalarExpression booleanFunction(String functionName, ScalarExpression expr1, ScalarExpression expr2)
    {
        ArrayList args = new ArrayList();
        args.add(expr1);
        args.add(expr2);
        return new SpatialBooleanExpression(new StringExpression(functionName, args), new CharacterLiteral(qs, null, 't'));
    }

    protected ScalarExpression booleanFunction(String functionName, ScalarExpression expr1, ScalarExpression expr2, ScalarExpression expr3)
    {
        ArrayList args = new ArrayList();
        args.add(expr1);
        args.add(expr2);
        args.add(expr3);
        return new SpatialBooleanExpression(new StringExpression(functionName, args), new CharacterLiteral(qs, null, 't'));
    }

}